@@ -17,20 +17,18 @@
 
 #include "BufferList.h"
 #include "LaTeX.h"
-#include "gettext.h"
 #include "LyXRC.h"
-#include "debug.h"
 #include "DepTable.h"
 
-#include "support/filetools.h"
+#include "support/debug.h"
 #include "support/convert.h"
+#include "support/FileName.h"
+#include "support/filetools.h"
+#include "support/gettext.h"
 #include "support/lstrings.h"
-#include "support/lyxlib.h"
 #include "support/Systemcall.h"
 #include "support/os.h"
 
-#include <boost/filesystem/operations.hpp>
-#include <boost/filesystem/path.hpp>
 #include <boost/regex.hpp>
 
 #include <fstream>
@@ -38,44 +36,12 @@
 using boost::regex;
 using boost::smatch;
 
-#ifndef CXX_GLOBAL_CSTD
-using std::sscanf;
-#endif
-
-using std::endl;
-using std::getline;
-using std::string;
-using std::ifstream;
-using std::set;
-using std::vector;
-
+using namespace std;
+using namespace lyx::support;
 
 namespace lyx {
 
-using support::absolutePath;
-using support::bformat;
-using support::changeExtension;
-using support::contains;
-using support::doesFileExist;
-using support::FileName;
-using support::findtexfile;
-using support::getcwd;
-using support::makeAbsPath;
-using support::onlyFilename;
-using support::prefixIs;
-using support::quoteName;
-using support::removeExtension;
-using support::rtrim;
-using support::rsplit;
-using support::split;
-using support::subst;
-using support::suffixIs;
-using support::Systemcall;
-using support::unlink;
-using support::trim;
-
 namespace os = support::os;
-namespace fs = boost::filesystem;
 
 // TODO: in no particular order
 // - get rid of the call to
@@ -152,29 +118,29 @@ void LaTeX::deleteFilesOnError() const
 	// What files do we have to delete?
 
 	// This will at least make latex do all the runs
-	unlink(depfile);
+	depfile.removeFile();
 
 	// but the reason for the error might be in a generated file...
 
 	// bibtex file
 	FileName const bbl(changeExtension(file.absFilename(), ".bbl"));
-	unlink(bbl);
+	bbl.removeFile();
 
 	// makeindex file
 	FileName const ind(changeExtension(file.absFilename(), ".ind"));
-	unlink(ind);
+	ind.removeFile();
 
 	// nomencl file
 	FileName const nls(changeExtension(file.absFilename(), ".nls"));
-	unlink(nls);
+	nls.removeFile();
 
 	// nomencl file (old version of the package)
 	FileName const gls(changeExtension(file.absFilename(), ".gls"));
-	unlink(gls);
+	gls.removeFile();
 
 	// Also remove the aux file
 	FileName const aux(changeExtension(file.absFilename(), ".aux"));
-	unlink(aux);
+	aux.removeFile();
 }
 
 
@@ -193,7 +159,7 @@ int LaTeX::run(TeXErrors & terr)
 	bool rerun = false; // rerun requested
 
 	// The class LaTeX does not know the temp path.
-	theBufferList().updateIncludedTeXfiles(getcwd().absFilename(),
+	theBufferList().updateIncludedTeXfiles(FileName::getcwd().absFilename(),
 		runparams);
 
 	// Never write the depfile if an error was encountered.
@@ -214,12 +180,12 @@ int LaTeX::run(TeXErrors & terr)
 	//             remake the dependency file.
 	//
 
-	bool had_depfile = doesFileExist(depfile);
+	bool had_depfile = depfile.exists();
 	bool run_bibtex = false;
 	FileName const aux_file(changeExtension(file.absFilename(), "aux"));
 
 	if (had_depfile) {
-		LYXERR(Debug::DEPEND) << "Dependency file exists" << endl;
+		LYXERR(Debug::DEPEND, "Dependency file exists");
 		// Read the dep file:
 		had_depfile = head.read(depfile);
 	}
@@ -231,24 +197,21 @@ int LaTeX::run(TeXErrors & terr)
 		// have aborted on error last time... in which cas we need
 		// to re-run latex and collect the error messages
 		// (even if they are the same).
-		if (!doesFileExist(output_file)) {
-			LYXERR(Debug::DEPEND)
-				<< "re-running LaTeX because output file doesn't exist."
-				<< endl;
+		if (!output_file.exists()) {
+			LYXERR(Debug::DEPEND,
+				"re-running LaTeX because output file doesn't exist.");
 		} else if (!head.sumchange()) {
-			LYXERR(Debug::DEPEND) << "return no_change" << endl;
+			LYXERR(Debug::DEPEND, "return no_change");
 			return NO_CHANGE;
 		} else {
-			LYXERR(Debug::DEPEND)
-				<< "Dependency file has changed" << endl;
+			LYXERR(Debug::DEPEND, "Dependency file has changed");
 		}
 
 		if (head.extchanged(".bib") || head.extchanged(".bst"))
 			run_bibtex = true;
 	} else
-		LYXERR(Debug::DEPEND)
-			<< "Dependency file does not exist, or has wrong format"
-			<< endl;
+		LYXERR(Debug::DEPEND,
+			"Dependency file does not exist, or has wrong format");
 
 	/// We scan the aux file even when had_depfile = false,
 	/// because we can run pdflatex on the file after running latex on it,
@@ -258,13 +221,13 @@ int LaTeX::run(TeXErrors & terr)
 		bibtex_info_old = scanAuxFiles(aux_file);
 
 	++count;
-	LYXERR(Debug::LATEX) << "Run #" << count << endl;
+	LYXERR(Debug::LATEX, "Run #" << count);
 	message(runMessage(count));
 
 	startscript();
 	scanres = scanLogFile(terr);
 	if (scanres & ERROR_RERUN) {
-		LYXERR(Debug::LATEX) << "Rerunning LaTeX" << endl;
+		LYXERR(Debug::LATEX, "Rerunning LaTeX");
 		startscript();
 		scanres = scanLogFile(terr);
 	}
@@ -292,24 +255,23 @@ int LaTeX::run(TeXErrors & terr)
 	// memoir (at least) writes an empty *idx file in the first place.
 	// A second latex run is needed.
 	FileName const idxfile(changeExtension(file.absFilename(), ".idx"));
-	rerun = doesFileExist(idxfile) &&
-		fs::is_empty(idxfile.toFilesystemEncoding());
+	rerun = idxfile.exists() && idxfile.isFileEmpty();
 
 	// run makeindex
 	if (head.haschanged(idxfile)) {
 		// no checks for now
-		LYXERR(Debug::LATEX) << "Running MakeIndex." << endl;
-		message(_("Running MakeIndex."));
+		LYXERR(Debug::LATEX, "Running MakeIndex.");
+		message(_("Running Index Processor."));
 		// onlyFilename() is needed for cygwin
 		rerun |= runMakeIndex(onlyFilename(idxfile.absFilename()),
 				runparams);
 	}
 	FileName const nlofile(changeExtension(file.absFilename(), ".nlo"));
 	if (head.haschanged(nlofile))
-		rerun |= runMakeIndexNomencl(file, runparams, ".nlo", ".nls");
+		rerun |= runMakeIndexNomencl(file, ".nlo", ".nls");
 	FileName const glofile(changeExtension(file.absFilename(), ".glo"));
 	if (head.haschanged(glofile))
-		rerun |= runMakeIndexNomencl(file, runparams, ".glo", ".gls");
+		rerun |= runMakeIndexNomencl(file, ".glo", ".gls");
 
 	// run bibtex
 	// if (scanres & UNDEF_CIT || scanres & RERUN || run_bibtex)
@@ -318,7 +280,7 @@ int LaTeX::run(TeXErrors & terr)
 		// "\bibdata" and/or "\bibstyle". If one of those
 		// tags is found -> run bibtex and set rerun = true;
 		// no checks for now
-		LYXERR(Debug::LATEX) << "Running BibTeX." << endl;
+		LYXERR(Debug::LATEX, "Running BibTeX.");
 		message(_("Running BibTeX."));
 		updateBibtexDependencies(head, bibtex_info);
 		rerun |= runBibTeX(bibtex_info);
@@ -344,11 +306,8 @@ int LaTeX::run(TeXErrors & terr)
 	if (rerun || head.sumchange()) {
 		rerun = false;
 		++count;
-		LYXERR(Debug::DEPEND)
-			<< "Dep. file has changed or rerun requested"
-			<< endl;
-		LYXERR(Debug::LATEX)
-			<< "Run #" << count << endl;
+		LYXERR(Debug::DEPEND, "Dep. file has changed or rerun requested");
+		LYXERR(Debug::LATEX, "Run #" << count);
 		message(runMessage(count));
 		startscript();
 		scanres = scanLogFile(terr);
@@ -361,9 +320,7 @@ int LaTeX::run(TeXErrors & terr)
 		deplog(head); // reads the latex log
 		head.update();
 	} else {
-		LYXERR(Debug::DEPEND)
-			<< "Dep. file has NOT changed"
-			<< endl;
+		LYXERR(Debug::DEPEND, "Dep. file has NOT changed");
 	}
 
 	// 1.5
@@ -377,8 +334,8 @@ int LaTeX::run(TeXErrors & terr)
 	// run makeindex if the <file>.idx has changed or was generated.
 	if (head.haschanged(idxfile)) {
 		// no checks for now
-		LYXERR(Debug::LATEX) << "Running MakeIndex." << endl;
-		message(_("Running MakeIndex."));
+		LYXERR(Debug::LATEX, "Running MakeIndex.");
+		message(_("Running Index Processor."));
 		// onlyFilename() is needed for cygwin
 		rerun = runMakeIndex(onlyFilename(changeExtension(
 				file.absFilename(), ".idx")), runparams);
@@ -386,9 +343,9 @@ int LaTeX::run(TeXErrors & terr)
 
 	// I am not pretty sure if need this twice.
 	if (head.haschanged(nlofile))
-		rerun |= runMakeIndexNomencl(file, runparams, ".nlo", ".nls");
+		rerun |= runMakeIndexNomencl(file, ".nlo", ".nls");
 	if (head.haschanged(glofile))
-		rerun |= runMakeIndexNomencl(file, runparams, ".glo", ".gls");
+		rerun |= runMakeIndexNomencl(file, ".glo", ".gls");
 
 	// 2
 	// we will only run latex more if the log file asks for it.
@@ -406,7 +363,7 @@ int LaTeX::run(TeXErrors & terr)
 		// MAX_RUNS are reached.
 		rerun = false;
 		++count;
-		LYXERR(Debug::LATEX) << "Run #" << count << endl;
+		LYXERR(Debug::LATEX, "Run #" << count);
 		message(runMessage(count));
 		startscript();
 		scanres = scanLogFile(terr);
@@ -421,7 +378,7 @@ int LaTeX::run(TeXErrors & terr)
 
 	// Write the dependencies to file.
 	head.write(depfile);
-	LYXERR(Debug::LATEX) << "Done." << endl;
+	LYXERR(Debug::LATEX, "Done.");
 	return scanres;
 }
 
@@ -440,9 +397,8 @@ int LaTeX::startscript()
 bool LaTeX::runMakeIndex(string const & f, OutputParams const & runparams,
 			 string const & params)
 {
-	LYXERR(Debug::LATEX)
-		<< "idx file has been made, running makeindex on file "
-		<< f << endl;
+	LYXERR(Debug::LATEX,
+		"idx file has been made, running makeindex on file " << f);
 	string tmp = lyxrc.index_command + ' ';
 
 	tmp = subst(tmp, "$$lang", runparams.document_language);
@@ -455,10 +411,9 @@ bool LaTeX::runMakeIndex(string const & f, OutputParams const & runparams,
 
 
 bool LaTeX::runMakeIndexNomencl(FileName const & file,
-		OutputParams const & runparams,
 		string const & nlo, string const & nls)
 {
-	LYXERR(Debug::LATEX) << "Running MakeIndex for nomencl." << endl;
+	LYXERR(Debug::LATEX, "Running MakeIndex for nomencl.");
 	message(_("Running MakeIndex for nomencl."));
 	string tmp = lyxrc.nomencl_command + ' ';
 	// onlyFilename() is needed for cygwin
@@ -483,7 +438,7 @@ LaTeX::scanAuxFiles(FileName const & file)
 		FileName const file2(basename
 			+ '.' + convert<string>(i)
 			+ ".aux");
-		if (!doesFileExist(file2))
+		if (!file2.exists())
 			break;
 		result.push_back(scanAuxFile(file2));
 	}
@@ -502,7 +457,7 @@ Aux_Info const LaTeX::scanAuxFile(FileName const & file)
 
 void LaTeX::scanAuxFile(FileName const & file, Aux_Info & aux_info)
 {
-	LYXERR(Debug::LATEX) << "Scanning aux file: " << file << endl;
+	LYXERR(Debug::LATEX, "Scanning aux file: " << file);
 
 	ifstream ifs(file.toFilesystemEncoding().c_str());
 	string token;
@@ -522,8 +477,7 @@ void LaTeX::scanAuxFile(FileName const & file, Aux_Info & aux_info)
 			while (!data.empty()) {
 				string citation;
 				data = split(data, citation, ',');
-				LYXERR(Debug::LATEX) << "Citation: "
-						     << citation << endl;
+				LYXERR(Debug::LATEX, "Citation: " << citation);
 				aux_info.citations.insert(citation);
 			}
 		} else if (regex_match(token, sub, reg2)) {
@@ -534,8 +488,7 @@ void LaTeX::scanAuxFile(FileName const & file, Aux_Info & aux_info)
 				string database;
 				data = split(data, database, ',');
 				database = changeExtension(database, "bib");
-				LYXERR(Debug::LATEX) << "BibTeX database: `"
-						     << database << '\'' << endl;
+				LYXERR(Debug::LATEX, "BibTeX database: `" << database << '\'');
 				aux_info.databases.insert(database);
 			}
 		} else if (regex_match(token, sub, reg3)) {
@@ -543,8 +496,7 @@ void LaTeX::scanAuxFile(FileName const & file, Aux_Info & aux_info)
 			// token is now the style file
 			// pass it to the helper
 			style = changeExtension(style, "bst");
-			LYXERR(Debug::LATEX) << "BibTeX style: `"
-					     << style << '\'' << endl;
+			LYXERR(Debug::LATEX, "BibTeX style: `" << style << '\'');
 			aux_info.styles.insert(style);
 		} else if (regex_match(token, sub, reg4)) {
 			string const file2 = sub.str(1);
@@ -610,7 +562,7 @@ int LaTeX::scanLogFile(TeXErrors & terr)
 	int retval = NO_ERRORS;
 	string tmp =
 		onlyFilename(changeExtension(file.absFilename(), ".log"));
-	LYXERR(Debug::LATEX) << "Log file: " << tmp << endl;
+	LYXERR(Debug::LATEX, "Log file: " << tmp);
 	FileName const fn = FileName(makeAbsPath(tmp));
 	ifstream ifs(fn.toFilesystemEncoding().c_str());
 	bool fle_style = false;
@@ -626,7 +578,7 @@ int LaTeX::scanLogFile(TeXErrors & terr)
 		token = subst(token, "\r", "");
 		smatch sub;
 
-		LYXERR(Debug::LATEX) << "Log line: " << token << endl;
+		LYXERR(Debug::LATEX, "Log line: " << token);
 
 		if (token.empty())
 			continue;
@@ -639,18 +591,16 @@ int LaTeX::scanLogFile(TeXErrors & terr)
 			// Here shall we handle different
 			// types of warnings
 			retval |= LATEX_WARNING;
-			LYXERR(Debug::LATEX) << "LaTeX Warning." << endl;
+			LYXERR(Debug::LATEX, "LaTeX Warning.");
 			if (contains(token, "Rerun to get cross-references")) {
 				retval |= RERUN;
-				LYXERR(Debug::LATEX)
-					<< "We should rerun." << endl;
+				LYXERR(Debug::LATEX, "We should rerun.");
 			// package clefval needs 2 latex runs before bibtex
 			} else if (contains(token, "Value of")
 				   && contains(token, "on page")
 				   && contains(token, "undefined")) {
 				retval |= ERROR_RERUN;
-				LYXERR(Debug::LATEX)
-					<< "Force rerun." << endl;
+				LYXERR(Debug::LATEX, "Force rerun.");
 			} else if (contains(token, "Citation")
 				   && contains(token, "on page")
 				   && contains(token, "undefined")) {
@@ -672,16 +622,14 @@ int LaTeX::scanLogFile(TeXErrors & terr)
 				   contains(token, "Rerun to get")) {
 				// at least longtable.sty and bibtopic.sty
 				// might use this.
-				LYXERR(Debug::LATEX)
-					<< "We should rerun." << endl;
+				LYXERR(Debug::LATEX, "We should rerun.");
 				retval |= RERUN;
 			}
 		} else if (token[0] == '(') {
 			if (contains(token, "Rerun LaTeX") ||
 			    contains(token, "Rerun to get")) {
 				// Used by natbib
-				LYXERR(Debug::LATEX)
-					<< "We should rerun." << endl;
+				LYXERR(Debug::LATEX, "We should rerun.");
 				retval |= RERUN;
 			}
 		} else if (prefixIs(token, "! ") ||
@@ -729,10 +677,8 @@ int LaTeX::scanLogFile(TeXErrors & terr)
 					errstr += "\n";
 					getline(ifs, tmp);
 				}
-				LYXERR(Debug::LATEX)
-					<< "line: " << line << '\n'
-					<< "Desc: " << desc << '\n'
-					<< "Text: " << errstr << endl;
+				LYXERR(Debug::LATEX, "line: " << line << '\n'
+					<< "Desc: " << desc << '\n' << "Text: " << errstr);
 				if (line == last_line)
 					++line_count;
 				else {
@@ -773,7 +719,7 @@ int LaTeX::scanLogFile(TeXErrors & terr)
 			}
 		}
 	}
-	LYXERR(Debug::LATEX) << "Log line: " << token << endl;
+	LYXERR(Debug::LATEX, "Log line: " << token);
 	return retval;
 }
 
@@ -782,8 +728,7 @@ namespace {
 
 bool insertIfExists(FileName const & absname, DepTable & head)
 {
-	if (doesFileExist(absname) &&
-	    !fs::is_directory(absname.toFilesystemEncoding())) {
+	if (absname.exists() && !absname.isDirectory()) {
 		head.insert(absname, true);
 		return true;
 	}
@@ -796,7 +741,7 @@ bool handleFoundFile(string const & ff, DepTable & head)
 	// convert from native os path to unix path
 	string foundfile = os::internal_path(trim(ff));
 
-	LYXERR(Debug::DEPEND) << "Found file: " << foundfile << endl;
+	LYXERR(Debug::DEPEND, "Found file: " << foundfile);
 
 	// Ok now we found a file.
 	// Now we should make sure that this is a file that we can
@@ -807,13 +752,12 @@ bool handleFoundFile(string const & ff, DepTable & head)
 	// (1) foundfile is an
 	//     absolute path and should
 	//     be inserted.
-	if (absolutePath(foundfile)) {
-		LYXERR(Debug::DEPEND) << "AbsolutePath file: "
-				      << foundfile << endl;
+	FileName absname(foundfile);
+	if (absname.isAbsolute()) {
+		LYXERR(Debug::DEPEND, "AbsolutePath file: " << foundfile);
 		// On initial insert we want to do the update at once
 		// since this file cannot be a file generated by
 		// the latex run.
-		FileName absname(foundfile);
 		if (!insertIfExists(absname, head)) {
 			// check for spaces
 			string strippedfile = foundfile;
@@ -836,11 +780,11 @@ bool handleFoundFile(string const & ff, DepTable & head)
 	}
 
 	string onlyfile = onlyFilename(foundfile);
-	FileName absname(makeAbsPath(onlyfile));
+	absname = makeAbsPath(onlyfile);
 
 	// check for spaces
 	while (contains(foundfile, ' ')) {
-		if (doesFileExist(absname))
+		if (absname.exists())
 			// everything o.k.
 			break;
 		else {
@@ -848,7 +792,7 @@ bool handleFoundFile(string const & ff, DepTable & head)
 			// marks; those have to be removed
 			string unquoted = subst(foundfile, "\"", "");
 			absname = makeAbsPath(unquoted);
-			if (doesFileExist(absname))
+			if (absname.exists())
 				break;
 			// strip off part after last space and try again
 			string strippedfile;
@@ -862,38 +806,26 @@ bool handleFoundFile(string const & ff, DepTable & head)
 
 	// (2) foundfile is in the tmpdir
 	//     insert it into head
-	if (doesFileExist(absname) &&
-	    !fs::is_directory(absname.toFilesystemEncoding())) {
+	if (absname.exists() && !absname.isDirectory()) {
 		// FIXME: This regex contained glo, but glo is used by the old
 		// version of nomencl.sty. Do we need to put it back?
-		static regex unwanted("^.*\\.(aux|log|dvi|bbl|ind)$");
+		static regex const unwanted("^.*\\.(aux|log|dvi|bbl|ind)$");
 		if (regex_match(onlyfile, unwanted)) {
-			LYXERR(Debug::DEPEND)
-				<< "We don't want "
-				<< onlyfile
-				<< " in the dep file"
-				<< endl;
+			LYXERR(Debug::DEPEND, "We don't want " << onlyfile
+				<< " in the dep file");
 		} else if (suffixIs(onlyfile, ".tex")) {
 			// This is a tex file generated by LyX
 			// and latex is not likely to change this
 			// during its runs.
-			LYXERR(Debug::DEPEND)
-				<< "Tmpdir TeX file: "
-				<< onlyfile
-				<< endl;
+			LYXERR(Debug::DEPEND, "Tmpdir TeX file: " << onlyfile);
 			head.insert(absname, true);
 		} else {
-			LYXERR(Debug::DEPEND)
-				<< "In tmpdir file:"
-				<< onlyfile
-				<< endl;
+			LYXERR(Debug::DEPEND, "In tmpdir file:" << onlyfile);
 			head.insert(absname);
 		}
 		return true;
 	} else {
-		LYXERR(Debug::DEPEND)
-			<< "Not a file or we are unable to find it."
-			<< endl;
+		LYXERR(Debug::DEPEND, "Not a file or we are unable to find it.");
 		return false;
 	}
 }
@@ -901,12 +833,11 @@ bool handleFoundFile(string const & ff, DepTable & head)
 
 bool checkLineBreak(string const & ff, DepTable & head)
 {
-	if (contains(ff, '.'))
-		// if we have a dot, we let handleFoundFile decide
-		return handleFoundFile(ff, head);
-	else
-		// else, we suspect a line break
+	if (!contains(ff, '.'))
 		return false;
+
+	// if we have a dot, we let handleFoundFile decide
+	return handleFoundFile(ff, head);
 }
 
 } // anon namespace
@@ -921,28 +852,28 @@ void LaTeX::deplog(DepTable & head)
 	string const logfile =
 		onlyFilename(changeExtension(file.absFilename(), ".log"));
 
-	static regex reg1("File: (.+).*");
-	static regex reg2("No file (.+)(.).*");
-	static regex reg3("\\\\openout[0-9]+.*=.*`(.+)(..).*");
+	static regex const reg1("File: (.+).*");
+	static regex const reg2("No file (.+)(.).*");
+	static regex const reg3("\\\\openout[0-9]+.*=.*`(.+)(..).*");
 	// If an index should be created, MikTex does not write a line like
 	//    \openout# = 'sample.idx'.
 	// but instead only a line like this into the log:
 	//   Writing index file sample.idx
-	static regex reg4("Writing index file (.+).*");
+	static regex const reg4("Writing index file (.+).*");
 	// files also can be enclosed in <...>
-	static regex reg5("<([^>]+)(.).*");
-	static regex regoldnomencl("Writing glossary file (.+).*");
-	static regex regnomencl("Writing nomenclature file (.+).*");
+	static regex const reg5("<([^>]+)(.).*");
+	static regex const regoldnomencl("Writing glossary file (.+).*");
+	static regex const regnomencl("Writing nomenclature file (.+).*");
 	// If a toc should be created, MikTex does not write a line like
 	//    \openout# = `sample.toc'.
 	// but only a line like this into the log:
 	//    \tf@toc=\write#
 	// This line is also written by tetex.
 	// This line is not present if no toc should be created.
-	static regex miktexTocReg("\\\\tf@toc=\\\\write.*");
-	static regex reg6(".*\\([^)]+.*");
+	static regex const miktexTocReg("\\\\tf@toc=\\\\write.*");
+	static regex const reg6(".*\\([^)]+.*");
 
-	FileName const fn(makeAbsPath(logfile));
+	FileName const fn = makeAbsPath(logfile);
 	ifstream ifs(fn.toFilesystemEncoding().c_str());
 	string lastline;
 	while (ifs) {
@@ -969,8 +900,8 @@ void LaTeX::deplog(DepTable & head)
 		// Here we exclude some cases where we are sure
 		// that there is no continued filename
 		if (!lastline.empty()) {
-			static regex package_info("Package \\w+ Info: .*");
-			static regex package_warning("Package \\w+ Warning: .*");
+			static regex const package_info("Package \\w+ Info: .*");
+			static regex const package_warning("Package \\w+ Warning: .*");
 			if (prefixIs(token, "File:") || prefixIs(token, "(Font)")
 			    || prefixIs(token, "Package:")
 			    || prefixIs(token, "Language:")
