@@ -14,22 +14,21 @@
 
 #include "InsetSpecialChar.h"
 
-#include "debug.h"
+#include "Dimension.h"
 #include "LaTeXFeatures.h"
-#include "Color.h"
 #include "Lexer.h"
 #include "MetricsInfo.h"
 
 #include "frontends/FontMetrics.h"
 #include "frontends/Painter.h"
 
+#include "support/debug.h"
+#include "support/docstream.h"
+
+using namespace std;
 
 namespace lyx {
 
-using std::string;
-using std::auto_ptr;
-using std::ostream;
-
 
 InsetSpecialChar::InsetSpecialChar(Kind k)
 	: kind_(k)
@@ -42,7 +41,7 @@ InsetSpecialChar::Kind InsetSpecialChar::kind() const
 }
 
 
-bool InsetSpecialChar::metrics(MetricsInfo & mi, Dimension & dim) const
+void InsetSpecialChar::metrics(MetricsInfo & mi, Dimension & dim) const
 {
 	frontend::FontMetrics const & fm =
 		theFontMetrics(mi.base.font);
@@ -51,48 +50,63 @@ bool InsetSpecialChar::metrics(MetricsInfo & mi, Dimension & dim) const
 
 	string s;
 	switch (kind_) {
-		case LIGATURE_BREAK:      s = "|";     break;
-		case END_OF_SENTENCE:     s = ".";     break;
-		case LDOTS:               s = ". . ."; break;
-		case MENU_SEPARATOR:      s = " x ";   break;
-		case HYPHENATION:      s = "-";   break;
+		case LIGATURE_BREAK:
+			s = "|";
+			break;
+		case END_OF_SENTENCE:
+			s = ".";
+			break;
+		case LDOTS:
+			s = ". . .";
+			break;
+		case MENU_SEPARATOR:
+			s = " x ";
+			break;
+		case HYPHENATION:
+			s = "-";
+			break;
+		case SLASH:
+			s = "/";
+			break;
+		case NOBREAKDASH:
+			s = "-";
+			break;
 	}
 	docstring ds(s.begin(), s.end());
 	dim.wid = fm.width(ds);
 	if (kind_ == HYPHENATION && dim.wid > 5)
 		dim.wid -= 2; // to make it look shorter
-	bool const changed = dim_ != dim;
-	dim_ = dim;
-	return changed;
+	
+	setDimCache(mi, dim);
 }
 
 
 void InsetSpecialChar::draw(PainterInfo & pi, int x, int y) const
 {
-	Font font = pi.base.font;
+	FontInfo font = pi.base.font;
 
 	switch (kind_) {
 	case HYPHENATION:
 	{
-		font.setColor(Color::special);
+		font.setColor(Color_special);
 		pi.pain.text(x, y, char_type('-'), font);
 		break;
 	}
 	case LIGATURE_BREAK:
 	{
-		font.setColor(Color::special);
+		font.setColor(Color_special);
 		pi.pain.text(x, y, char_type('|'), font);
 		break;
 	}
 	case END_OF_SENTENCE:
 	{
-		font.setColor(Color::special);
+		font.setColor(Color_special);
 		pi.pain.text(x, y, char_type('.'), font);
 		break;
 	}
 	case LDOTS:
 	{
-		font.setColor(Color::special);
+		font.setColor(Color_special);
 		string ell = ". . . ";
 		docstring dell(ell.begin(), ell.end());
 		pi.pain.text(x, y, dell, font);
@@ -114,7 +128,19 @@ void InsetSpecialChar::draw(PainterInfo & pi, int x, int y) const
 		xp[2] = ox + w; yp[2] = y - h/2;
 		xp[3] = ox;     yp[3] = y;
 
-		pi.pain.lines(xp, yp, 4, Color::special);
+		pi.pain.lines(xp, yp, 4, Color_special);
+		break;
+	}
+	case SLASH:
+	{
+		font.setColor(Color_special);
+		pi.pain.text(x, y, char_type('/'), font);
+		break;
+	}
+	case NOBREAKDASH:
+	{
+		font.setColor(Color_latex);
+		pi.pain.text(x, y, char_type('-'), font);
 		break;
 	}
 	}
@@ -122,7 +148,7 @@ void InsetSpecialChar::draw(PainterInfo & pi, int x, int y) const
 
 
 // In lyxf3 this will be just LaTeX
-void InsetSpecialChar::write(Buffer const &, ostream & os) const
+void InsetSpecialChar::write(ostream & os) const
 {
 	string command;
 	switch (kind_) {
@@ -141,13 +167,19 @@ void InsetSpecialChar::write(Buffer const &, ostream & os) const
 	case MENU_SEPARATOR:
 		command = "\\menuseparator";
 		break;
+	case SLASH:
+		command = "\\slash{}";
+		break;
+	case NOBREAKDASH:
+		command = "\\nobreakdash-";
+		break;
 	}
 	os << "\\SpecialChar " << command << "\n";
 }
 
 
 // This function will not be necessary when lyx3
-void InsetSpecialChar::read(Buffer const &, Lexer & lex)
+void InsetSpecialChar::read(Lexer & lex)
 {
 	lex.next();
 	string const command = lex.getString();
@@ -162,13 +194,17 @@ void InsetSpecialChar::read(Buffer const &, Lexer & lex)
 		kind_ = LDOTS;
 	else if (command == "\\menuseparator")
 		kind_ = MENU_SEPARATOR;
+	else if (command == "\\slash{}")
+		kind_ = SLASH;
+	else if (command == "\\nobreakdash-")
+		kind_ = NOBREAKDASH;
 	else
 		lex.printError("InsetSpecialChar: Unknown kind: `$$Token'");
 }
 
 
-int InsetSpecialChar::latex(Buffer const &, odocstream & os,
-			    OutputParams const &) const
+int InsetSpecialChar::latex(odocstream & os,
+			    OutputParams const & rp) const
 {
 	switch (kind_) {
 	case HYPHENATION:
@@ -186,13 +222,20 @@ int InsetSpecialChar::latex(Buffer const &, odocstream & os,
 	case MENU_SEPARATOR:
 		os << "\\lyxarrow{}";
 		break;
+	case SLASH:
+		os << "\\slash{}";
+		break;
+	case NOBREAKDASH:
+		if (rp.moving_arg)
+			os << "\\protect";
+		os << "\\nobreakdash-";
+		break;
 	}
 	return 0;
 }
 
 
-int InsetSpecialChar::plaintext(Buffer const &, odocstream & os,
-				OutputParams const &) const
+int InsetSpecialChar::plaintext(odocstream & os, OutputParams const &) const
 {
 	switch (kind_) {
 	case HYPHENATION:
@@ -207,13 +250,18 @@ int InsetSpecialChar::plaintext(Buffer const &, odocstream & os,
 	case MENU_SEPARATOR:
 		os << "->";
 		return 2;
+	case SLASH:
+		os << '/';
+		return 1;
+	case NOBREAKDASH:
+		os << '-';
+		return 1;
 	}
 	return 0;
 }
 
 
-int InsetSpecialChar::docbook(Buffer const &, odocstream & os,
-			      OutputParams const &) const
+int InsetSpecialChar::docbook(odocstream & os, OutputParams const &) const
 {
 	switch (kind_) {
 	case HYPHENATION:
@@ -228,28 +276,29 @@ int InsetSpecialChar::docbook(Buffer const &, odocstream & os,
 	case MENU_SEPARATOR:
 		os << "&lyxarrow;";
 		break;
+	case SLASH:
+		os << '/';
+		break;
+	case NOBREAKDASH:
+		os << '-';
+		break;
 	}
 	return 0;
 }
 
 
-void InsetSpecialChar::textString(Buffer const & buf, odocstream & os) const
+void InsetSpecialChar::textString(odocstream & os) const
 {
-	plaintext(buf, os, OutputParams(0));
-}
-
-
-auto_ptr<Inset> InsetSpecialChar::doClone() const
-{
-	return auto_ptr<Inset>(new InsetSpecialChar(kind_));
+	plaintext(os, OutputParams(0));
 }
 
 
 void InsetSpecialChar::validate(LaTeXFeatures & features) const
 {
-	if (kind_ == MENU_SEPARATOR) {
+	if (kind_ == MENU_SEPARATOR)
 		features.require("lyxarrow");
-	}
+	if (kind_ == NOBREAKDASH)
+		features.require("amsmath");
 }
 
 
