@@ -14,41 +14,41 @@
 
 #include "Buffer.h"
 #include "BufferParams.h"
-#include "debug.h"
 #include "Encoding.h"
+#include "InsetList.h"
 #include "Language.h"
+#include "Layout.h"
 #include "LyXRC.h"
 #include "OutputParams.h"
 #include "Paragraph.h"
 #include "paragraph_funcs.h"
 #include "ParagraphParameters.h"
+#include "TextClass.h"
 #include "TexRow.h"
 #include "VSpace.h"
 
 #include "insets/InsetBibitem.h"
 #include "insets/InsetOptArg.h"
 
+#include "support/lassert.h"
+#include "support/debug.h"
 #include "support/lstrings.h"
 
+#include <boost/next_prior.hpp>
+
+using namespace std;
+using namespace lyx::support;
+
 
 namespace lyx {
 
-using support::subst;
-
-using std::endl;
-using std::string;
-using std::pair;
-using std::make_pair;
-
-
 namespace {
 
-
 enum OpenEncoding {
 		none,
 		inputenc,
 		CJK
-	};
+};
 
 static int open_encoding_ = none;
 static int cjk_inherited_ = 0;
@@ -77,14 +77,18 @@ TeXDeeper(Buffer const & buf,
 	  odocstream & os, TexRow & texrow,
 	  OutputParams const & runparams)
 {
-	LYXERR(Debug::LATEX) << "TeXDeeper...     " << &*pit << endl;
+	LYXERR(Debug::LATEX, "TeXDeeper...     " << &*pit);
 	ParagraphList::const_iterator par = pit;
 
 	ParagraphList const & paragraphs = text.paragraphs();
 
 	while (par != paragraphs.end() &&
-		     par->params().depth() == pit->params().depth()) {
-		if (par->layout()->isEnvironment()) {
+					par->params().depth() == pit->params().depth()) {
+		// FIXME This test should not be necessary.
+		// We should perhaps issue an error if it is.
+		Layout const & style = par->forcePlainLayout() ?
+			buf.params().documentClass().plainLayout() : par->layout();
+		if (style.isEnvironment()) {
 			par = TeXEnvironment(buf, text, par,
 					     os, texrow, runparams);
 		} else {
@@ -92,7 +96,7 @@ TeXDeeper(Buffer const & buf,
 					     os, texrow, runparams);
 		}
 	}
-	LYXERR(Debug::LATEX) << "TeXDeeper...done " << endl;
+	LYXERR(Debug::LATEX, "TeXDeeper...done ");
 
 	return par;
 }
@@ -105,11 +109,14 @@ TeXEnvironment(Buffer const & buf,
 	       odocstream & os, TexRow & texrow,
 	       OutputParams const & runparams)
 {
-	LYXERR(Debug::LATEX) << "TeXEnvironment...     " << &*pit << endl;
+	LYXERR(Debug::LATEX, "TeXEnvironment...     " << &*pit);
 
 	BufferParams const & bparams = buf.params();
 
-	Layout_ptr const & style = pit->layout();
+	// FIXME This test should not be necessary.
+	// We should perhaps issue an error if it is.
+	Layout const & style = pit->forcePlainLayout() ?
+		bparams.documentClass().plainLayout() : pit->layout();
 
 	ParagraphList const & paragraphs = text.paragraphs();
 
@@ -155,25 +162,25 @@ TeXEnvironment(Buffer const & buf,
 		leftindent_open = true;
 	}
 
-	if (style->isEnvironment()) {
-		os << "\\begin{" << from_ascii(style->latexname()) << '}';
-		if (style->optionalargs > 0) {
-			int ret = latexOptArgInsets(buf, *pit, os, runparams,
-						    style->optionalargs);
+	if (style.isEnvironment()) {
+		os << "\\begin{" << from_ascii(style.latexname()) << '}';
+		if (style.optionalargs > 0) {
+			int ret = latexOptArgInsets(*pit, os, runparams,
+						    style.optionalargs);
 			while (ret > 0) {
 				texrow.newline();
 				--ret;
 			}
 		}
-		if (style->latextype == LATEX_LIST_ENVIRONMENT) {
+		if (style.latextype == LATEX_LIST_ENVIRONMENT) {
 			os << '{'
 			   << pit->params().labelWidthString()
 			   << "}\n";
-		} else if (style->labeltype == LABEL_BIBLIO) {
+		} else if (style.labeltype == LABEL_BIBLIO) {
 			// ale970405
 			os << '{' << bibitemWidest(buf) << "}\n";
 		} else
-			os << from_ascii(style->latexparam()) << '\n';
+			os << from_ascii(style.latexparam()) << '\n';
 		texrow.newline();
 	}
 
@@ -182,7 +189,7 @@ TeXEnvironment(Buffer const & buf,
 	if (par_language->encoding()->package() == Encoding::CJK &&
 	    open_encoding_ != CJK && pit->isMultiLingual(bparams)) {
 		os << "\\begin{CJK}{" << from_ascii(par_language->encoding()->latexName())
-		   << "}{}%\n";
+		   << "}{" << from_ascii(bparams.fontsCJK) << "}%\n";
 		open_encoding_ = CJK;
 		cjk_nested = true;
 		texrow.newline();
@@ -199,12 +206,11 @@ TeXEnvironment(Buffer const & buf,
 			os << '\n';
 			texrow.newline();
 		} else if (par->params().depth() > pit->params().depth()) {
-			    if (par->layout()->isParagraph()) {
+			if (par->layout().isParagraph()) {
+			  // Thinko!
+			  // How to handle this? (Lgb)
+			  //&& !suffixIs(os, "\n\n")
 
-			    // Thinko!
-			    // How to handle this? (Lgb)
-			    //&& !suffixIs(os, "\n\n")
-				    //) {
 				// There should be at least one '\n' already
 				// but we need there to be two for Standard
 				// paragraphs that are depth-increment'ed to be
@@ -234,8 +240,8 @@ TeXEnvironment(Buffer const & buf,
 		open_encoding_ = none;
 	}
 
-	if (style->isEnvironment()) {
-		os << "\\end{" << from_ascii(style->latexname()) << "}\n";
+	if (style.isEnvironment()) {
+		os << "\\end{" << from_ascii(style.latexname()) << "}\n";
 		texrow.newline();
 	}
 
@@ -245,25 +251,26 @@ TeXEnvironment(Buffer const & buf,
 	}
 
 	if (par != paragraphs.end())
-		LYXERR(Debug::LATEX) << "TeXEnvironment...done " << &*par << endl;
+		LYXERR(Debug::LATEX, "TeXEnvironment...done " << &*par);
+
 	return par;
 }
 
-}
+} // namespace anon
 
 
-int latexOptArgInsets(Buffer const & buf, Paragraph const & par,
-		      odocstream & os, OutputParams const & runparams, int number)
+int latexOptArgInsets(Paragraph const & par, odocstream & os,
+	OutputParams const & runparams, int number)
 {
 	int lines = 0;
 
-	InsetList::const_iterator it = par.insetlist.begin();
-	InsetList::const_iterator end = par.insetlist.end();
+	InsetList::const_iterator it = par.insetList().begin();
+	InsetList::const_iterator end = par.insetList().end();
 	for (; it != end && number > 0 ; ++it) {
-		if (it->inset->lyxCode() == Inset::OPTARG_CODE) {
+		if (it->inset->lyxCode() == OPTARG_CODE) {
 			InsetOptArg * ins =
 				static_cast<InsetOptArg *>(it->inset);
-			lines += ins->latexOptional(buf, os, runparams);
+			lines += ins->latexOptional(os, runparams);
 			--number;
 		}
 	}
@@ -276,29 +283,47 @@ namespace {
 ParagraphList::const_iterator
 TeXOnePar(Buffer const & buf,
 	  Text const & text,
-	  ParagraphList::const_iterator pit,
+	  ParagraphList::const_iterator const pit,
 	  odocstream & os, TexRow & texrow,
 	  OutputParams const & runparams_in,
 	  string const & everypar)
 {
-	LYXERR(Debug::LATEX) << "TeXOnePar...     " << &*pit << " '"
-		<< everypar << "'" << endl;
+	LYXERR(Debug::LATEX, "TeXOnePar...     " << &*pit << " '"
+		<< everypar << "'");
 	BufferParams const & bparams = buf.params();
-	Layout_ptr style;
 	ParagraphList const & paragraphs = text.paragraphs();
 
-	bool const maintext = text.isMainText(buf);
-
-	// In an inset with unlimited length (all in one row),
-	// force layout to default
-	if (!pit->forceDefaultParagraphs())
-		style = pit->layout();
-	else
-		style = bparams.getTextClass().defaultLayout();
+	ParagraphList::const_iterator const priorpit = 
+		pit == paragraphs.begin() ? pit : boost::prior(pit);
+	ParagraphList::const_iterator const nextpit = 
+		pit == paragraphs.end() ? pit : boost::next(pit);
 
 	OutputParams runparams = runparams_in;
-	runparams.moving_arg |= style->needprotect;
+	runparams.isLastPar = nextpit == paragraphs.end();
 
+	if (runparams.verbatim) {
+		int const dist = distance(paragraphs.begin(), pit);
+		Font const outerfont = outerFont(dist, paragraphs);
+
+		// No newline if only one paragraph in this lyxtext
+		if (dist > 0) {
+			os << '\n';
+			texrow.newline();
+		}
+
+		/*bool need_par = */ pit->latex(bparams, outerfont,
+			os, texrow, runparams);
+		return nextpit;
+	}
+
+	// FIXME This check should not really be needed.
+	// Perhaps we should issue an error if it is.
+	Layout const style = pit->forcePlainLayout() ?
+		bparams.documentClass().plainLayout() : pit->layout();
+
+	runparams.moving_arg |= style.needprotect;
+
+	bool const maintext = text.isMainText(buf);
 	// we are at the beginning of an inset and CJK is already open;
 	// we count inheritation levels to get the inset nesting right.
 	if (pit == paragraphs.begin() && !maintext
@@ -311,9 +336,9 @@ TeXOnePar(Buffer const & buf,
 	Language const * const par_language = pit->getParLanguage(bparams);
 	// The document's language
 	Language const * const doc_language = bparams.language;
-	// The language that was in effect when the environment this paragraph is 
+	// The language that was in effect when the environment this paragraph is
 	// inside of was opened
-	Language const * const outer_language = 
+	Language const * const outer_language =
 		(runparams.local_font != 0) ?
 			runparams.local_font->language() : doc_language;
 	// The previous language that was in effect is either the language of
@@ -321,15 +346,15 @@ TeXOnePar(Buffer const & buf,
 	// if there is no previous paragraph
 	Language const * const prev_language =
 		(pit != paragraphs.begin()) ?
-			boost::prior(pit)->getParLanguage(bparams) : outer_language;
+			priorpit->getParLanguage(bparams) : outer_language;
 
 	if (par_language->babel() != prev_language->babel()
 	    // check if we already put language command in TeXEnvironment()
-	    && !(style->isEnvironment()
+	    && !(style.isEnvironment()
 		 && (pit == paragraphs.begin() ||
-		     (boost::prior(pit)->layout() != pit->layout() &&
-		      boost::prior(pit)->getDepth() <= pit->getDepth())
-		     || boost::prior(pit)->getDepth() < pit->getDepth())))
+		     (priorpit->layout() != pit->layout() &&
+		      priorpit->getDepth() <= pit->getDepth())
+		     || priorpit->getDepth() < pit->getDepth())))
 	{
 		if (!lyxrc.language_command_end.empty() &&
 		    prev_language->babel() != outer_language->babel() &&
@@ -343,7 +368,7 @@ TeXOnePar(Buffer const & buf,
 			texrow.newline();
 		}
 
-		// We need to open a new language if we couldn't close the previous 
+		// We need to open a new language if we couldn't close the previous
 		// one (because there's no language_command_end); and even if we closed
 		// the previous one, if the current language is different than the
 		// outer_language (which is currently in effect once the previous one
@@ -362,11 +387,11 @@ TeXOnePar(Buffer const & buf,
 					// are we in an inset?
 					runparams.local_font != 0 &&
 					// is the inset within an \L or \R?
-					// 
+					//
 					// FIXME: currently, we don't check this; this means that
-					// we'll have unnnecessary \L and \R commands, but that 
+					// we'll have unnnecessary \L and \R commands, but that
 					// doesn't seem to hurt (though latex will complain)
-					// 
+					//
 					// is this paragraph in the opposite direction?
 					runparams.local_font->isRightToLeft() !=
 						par_language->rightToLeft()
@@ -436,7 +461,7 @@ TeXOnePar(Buffer const & buf,
 				if (par_language->encoding()->package() == Encoding::CJK &&
 				    open_encoding_ != CJK && cjk_inherited_ == 0) {
 					os << "\\begin{CJK}{" << from_ascii(par_language->encoding()->latexName())
-					   << "}{}%\n";
+					   << "}{" << from_ascii(bparams.fontsCJK) << "}%\n";
 					open_encoding_ = CJK;
 					texrow.newline();
 				}
@@ -463,10 +488,8 @@ TeXOnePar(Buffer const & buf,
 		}
 	}
 
-	// In an inset with unlimited length (all in one row),
-	// don't allow any special options in the paragraph
-	bool const useSetSpace = bparams.getTextClass().provides("SetSpace");
-	if (!pit->forceDefaultParagraphs()) {
+	bool const useSetSpace = bparams.documentClass().provides("SetSpace");
+	if (pit->allowParagraphCustomization()) {
 		if (pit->params().startOfAppendix()) {
 			os << "\\appendix\n";
 			texrow.newline();
@@ -474,34 +497,34 @@ TeXOnePar(Buffer const & buf,
 
 		if (!pit->params().spacing().isDefault()
 			&& (pit == paragraphs.begin()
-			    || !boost::prior(pit)->hasSameLayout(*pit)))
+			    || !priorpit->hasSameLayout(*pit)))
 		{
 			os << from_ascii(pit->params().spacing().writeEnvirBegin(useSetSpace))
 			    << '\n';
 			texrow.newline();
 		}
 
-		if (style->isCommand()) {
+		if (style.isCommand()) {
 			os << '\n';
 			texrow.newline();
 		}
 	}
 
-	switch (style->latextype) {
+	switch (style.latextype) {
 	case LATEX_COMMAND:
-		os << '\\' << from_ascii(style->latexname());
+		os << '\\' << from_ascii(style.latexname());
 
 		// Separate handling of optional argument inset.
-		if (style->optionalargs > 0) {
-			int ret = latexOptArgInsets(buf, *pit, os, runparams,
-						    style->optionalargs);
+		if (style.optionalargs > 0) {
+			int ret = latexOptArgInsets(*pit, os, runparams,
+						    style.optionalargs);
 			while (ret > 0) {
 				texrow.newline();
 				--ret;
 			}
 		}
 		else
-			os << from_ascii(style->latexparam());
+			os << from_ascii(style.latexparam());
 		break;
 	case LATEX_ITEM_ENVIRONMENT:
 	case LATEX_LIST_ENVIRONMENT:
@@ -514,13 +537,12 @@ TeXOnePar(Buffer const & buf,
 		break;
 	}
 
-	Font const outerfont =
-		outerFont(std::distance(paragraphs.begin(), pit),
+	Font const outerfont = outerFont(distance(paragraphs.begin(), pit),
 			  paragraphs);
 
 	// FIXME UNICODE
 	os << from_utf8(everypar);
-	bool need_par = pit->simpleTeXOnePar(buf, bparams, outerfont,
+	bool need_par = pit->latex(bparams, outerfont,
 					     os, texrow, runparams);
 
 	// Make sure that \\par is done with the font of the last
@@ -533,15 +555,14 @@ TeXOnePar(Buffer const & buf,
 	// We do not need to use to change the font for the last paragraph
 	// or for a command.
 
-	Font const font =
-		(pit->empty()
+	Font const font = pit->empty()
 		 ? pit->getLayoutFont(bparams, outerfont)
-		 : pit->getFont(bparams, pit->size() - 1, outerfont));
+		 : pit->getFont(bparams, pit->size() - 1, outerfont);
 
-	bool is_command = style->isCommand();
+	bool const is_command = style.isCommand();
 
-	if (style->resfont.size() != font.size()
-	    && boost::next(pit) != paragraphs.end()
+	if (style.resfont.size() != font.fontInfo().size()
+	    && nextpit != paragraphs.end()
 	    && !is_command) {
 		if (!need_par)
 			os << '{';
@@ -552,35 +573,32 @@ TeXOnePar(Buffer const & buf,
 		os << '}';
 
 	bool pending_newline = false;
-	switch (style->latextype) {
+	switch (style.latextype) {
 	case LATEX_ITEM_ENVIRONMENT:
 	case LATEX_LIST_ENVIRONMENT:
-		if (boost::next(pit) != paragraphs.end()
-		    && (pit->params().depth() < boost::next(pit)->params().depth()))
+		if (nextpit != paragraphs.end()
+		    && (pit->params().depth() < nextpit->params().depth()))
 			pending_newline = true;
 		break;
 	case LATEX_ENVIRONMENT: {
 		// if its the last paragraph of the current environment
 		// skip it otherwise fall through
-		ParagraphList::const_iterator next = boost::next(pit);
-
-		if (next != paragraphs.end()
-		    && (next->layout() != pit->layout()
-			|| next->params().depth() != pit->params().depth()))
+		if (nextpit != paragraphs.end() && 
+		    (nextpit->layout() != pit->layout()
+		     || nextpit->params().depth() != pit->params().depth()))
 			break;
 	}
 
 		// fall through possible
 	default:
 		// we don't need it for the last paragraph!!!
-		if (boost::next(pit) != paragraphs.end())
+		if (nextpit != paragraphs.end())
 			pending_newline = true;
 	}
 
-	if (!pit->forceDefaultParagraphs()) {
+	if (pit->allowParagraphCustomization()) {
 		if (!pit->params().spacing().isDefault()
-			&& (boost::next(pit) == paragraphs.end()
-			    || !boost::next(pit)->hasSameLayout(*pit)))
+			&& (nextpit == paragraphs.end() || !nextpit->hasSameLayout(*pit)))
 		{
 			if (pending_newline) {
 				os << '\n';
@@ -594,7 +612,7 @@ TeXOnePar(Buffer const & buf,
 	// Closing the language is needed for the last paragraph; it is also
 	// needed if we're within an \L or \R that we may have opened above (not
 	// necessarily in this paragraph) and are about to close.
-	bool closing_rtl_ltr_environment = 
+	bool closing_rtl_ltr_environment =
 		// not for ArabTeX
 		(par_language->lang() != "arabic_arabtex" &&
 		 outer_language->lang() != "arabic_arabtex") &&
@@ -602,13 +620,13 @@ TeXOnePar(Buffer const & buf,
 		runparams.local_font != 0 &&
 		runparams.local_font->isRightToLeft() != par_language->rightToLeft() &&
 		// are we about to close the language?
-		((boost::next(pit) != paragraphs.end() &&
-		  par_language->babel() != 
-		  	(boost::next(pit)->getParLanguage(bparams))->babel()) ||
-		 (boost::next(pit) == paragraphs.end() &&
-		  par_language->babel() != outer_language->babel()));
+		((nextpit != paragraphs.end() &&
+		  par_language->babel() !=
+		    (nextpit->getParLanguage(bparams))->babel()) ||
+		  (nextpit == paragraphs.end() &&
+		    par_language->babel() != outer_language->babel()));
 
-	if (closing_rtl_ltr_environment || (boost::next(pit) == paragraphs.end()
+	if (closing_rtl_ltr_environment || (nextpit == paragraphs.end()
 	    && par_language->babel() != outer_language->babel())) {
 		// Since \selectlanguage write the language to the aux file,
 		// we need to reset the language at the end of footnote or
@@ -647,22 +665,22 @@ TeXOnePar(Buffer const & buf,
 
 	// if this is a CJK-paragraph and the next isn't, close CJK
 	// also if the next paragraph is a multilingual environment (because of nesting)
-	if (boost::next(pit) != paragraphs.end() && open_encoding_ == CJK &&
-	    (boost::next(pit)->getParLanguage(bparams)->encoding()->package() != Encoding::CJK ||
-	     boost::next(pit)->layout()->isEnvironment() && boost::next(pit)->isMultiLingual(bparams))
+	if (nextpit != paragraphs.end() && open_encoding_ == CJK &&
+	    (nextpit->getParLanguage(bparams)->encoding()->package() != Encoding::CJK ||
+	     nextpit->layout().isEnvironment() && nextpit->isMultiLingual(bparams))
 	     // in environments, CJK has to be closed later (nesting!)
-	     && !style->isEnvironment()) {
+	     && !style.isEnvironment()) {
 		os << "\\end{CJK}\n";
 		open_encoding_ = none;
 	}
 
 	// If this is the last paragraph, close the CJK environment
 	// if necessary. If it's an environment, we'll have to \end that first.
-	if (boost::next(pit) == paragraphs.end() && !style->isEnvironment()) {
+	if (nextpit == paragraphs.end() && !style.isEnvironment()) {
 		switch (open_encoding_) {
 			case CJK: {
 				// do nothing at the end of child documents
-				if (maintext && buf.getMasterBuffer() != &buf)
+				if (maintext && buf.masterBuffer() != &buf)
 					break;
 				// end of main text
 				if (maintext) {
@@ -691,29 +709,42 @@ TeXOnePar(Buffer const & buf,
 	// If this is the last paragraph, and a local_font was set upon entering
 	// the inset, and we're using "auto" or "default" encoding, the encoding
 	// should be set back to that local_font's encoding.
-	if (boost::next(pit) == paragraphs.end() && runparams_in.local_font != 0
+	if (nextpit == paragraphs.end() && runparams_in.local_font != 0
+	    && runparams_in.encoding != runparams_in.local_font->language()->encoding()
 	    && (bparams.inputenc == "auto" || bparams.inputenc == "default")) {
 		runparams_in.encoding = runparams_in.local_font->language()->encoding();
 		os << setEncoding(runparams_in.encoding->iconvName());
-
 	}
 	// Otherwise, the current encoding should be set for the next paragraph.
 	else
 		runparams_in.encoding = runparams.encoding;
 
 
-	// we don't need it for the last paragraph!!!
-	// Note from JMarc: we will re-add a \n explicitely in
+	// we don't need a newline for the last paragraph!!!
+	// Note from JMarc: we will re-add a \n explicitly in
 	// TeXEnvironment, because it is needed in this case
-	if (boost::next(pit) != paragraphs.end()) {
-		os << '\n';
-		texrow.newline();
+	if (nextpit != paragraphs.end()) {
+		Layout const & next_layout = nextpit->layout();
+		if (style == next_layout
+		    // no blank lines before environments!
+		    || !next_layout.isEnvironment()
+		    // unless there's a depth change
+		    // FIXME What we really want to do here is put every \begin and \end
+		    // tag on a new line (which was not the case with nested environments).
+		    // But in the present state of play, we don't have access to the
+		    // information whether the current TeX row is empty or not.
+		    // For some ideas about how to fix this, see this thread:
+		    // http://www.mail-archive.com/lyx-devel@lists.lyx.org/msg145787.html
+		    || nextpit->params().depth() != pit->params().depth()) {
+			os << '\n';
+			texrow.newline();
+		}
 	}
 
-	if (boost::next(pit) != paragraphs.end())
-		LYXERR(Debug::LATEX) << "TeXOnePar...done " << &*boost::next(pit) << endl;
+	if (nextpit != paragraphs.end())
+		LYXERR(Debug::LATEX, "TeXOnePar...done " << &*nextpit);
 
-	return ++pit;
+	return nextpit;
 }
 
 } // anon namespace
@@ -730,12 +761,12 @@ void latexParagraphs(Buffer const & buf,
 	bool was_title = false;
 	bool already_title = false;
 	BufferParams const & bparams = buf.params();
-	TextClass const & tclass = bparams.getTextClass();
+	DocumentClass const & tclass = bparams.documentClass();
 	ParagraphList const & paragraphs = text.paragraphs();
 	ParagraphList::const_iterator par = paragraphs.begin();
 	ParagraphList::const_iterator endpar = paragraphs.end();
 
-	BOOST_ASSERT(runparams.par_begin <= runparams.par_end);
+	LASSERT(runparams.par_begin <= runparams.par_end, /**/);
 	// if only part of the paragraphs will be outputed
 	if (runparams.par_begin !=  runparams.par_end) {
 		par = boost::next(paragraphs.begin(), runparams.par_begin);
@@ -747,7 +778,7 @@ void latexParagraphs(Buffer const & buf,
 	}
 
 	bool const maintext = text.isMainText(buf);
-	bool const is_child = buf.getMasterBuffer() != &buf;
+	bool const is_child = buf.masterBuffer() != &buf;
 
 	// Open a CJK environment at the beginning of the main buffer
 	// if the document's language is a CJK language
@@ -755,7 +786,7 @@ void latexParagraphs(Buffer const & buf,
 	if (maintext && !is_child
 	    && bparams.encoding().package() == Encoding::CJK) {
 		os << "\\begin{CJK}{" << from_ascii(bparams.encoding().latexName())
-		<< "}{}%\n";
+		<< "}{" << from_ascii(bparams.fontsCJK) << "}%\n";
 		texrow.newline();
 		open_encoding_ = CJK;
 	}
@@ -775,60 +806,52 @@ void latexParagraphs(Buffer const & buf,
 	// if only_body
 	while (par != endpar) {
 		lastpar = par;
-		// well we have to check if we are in an inset with unlimited
-		// length (all in one row) if that is true then we don't allow
-		// any special options in the paragraph and also we don't allow
-		// any environment other than the default layout of the
-		// text class to be valid!
-		if (!par->forceDefaultParagraphs()) {
-			Layout_ptr const & layout = par->layout();
+		// FIXME This check should not be needed. We should
+		// perhaps issue an error if it is.
+		Layout const & layout = par->forcePlainLayout() ?
+				tclass.plainLayout() :
+				par->layout();
 
-			if (layout->intitle) {
-				if (already_title) {
-					lyxerr << "Error in latexParagraphs: You"
-						" should not mix title layouts"
-						" with normal ones." << endl;
-				} else if (!was_title) {
-					was_title = true;
-					if (tclass.titletype() == TITLE_ENVIRONMENT) {
-						os << "\\begin{"
-						    << from_ascii(tclass.titlename())
-						    << "}\n";
-						texrow.newline();
-					}
-				}
-			} else if (was_title && !already_title) {
+		if (layout.intitle) {
+			if (already_title) {
+				lyxerr << "Error in latexParagraphs: You"
+					" should not mix title layouts"
+					" with normal ones." << endl;
+			} else if (!was_title) {
+				was_title = true;
 				if (tclass.titletype() == TITLE_ENVIRONMENT) {
-					os << "\\end{" << from_ascii(tclass.titlename())
-					    << "}\n";
+					os << "\\begin{"
+							<< from_ascii(tclass.titlename())
+							<< "}\n";
+					texrow.newline();
 				}
-				else {
-					os << "\\" << from_ascii(tclass.titlename())
-					    << "\n";
-				}
-				texrow.newline();
-				already_title = true;
-				was_title = false;
 			}
+		} else if (was_title && !already_title) {
+			if (tclass.titletype() == TITLE_ENVIRONMENT) {
+				os << "\\end{" << from_ascii(tclass.titlename())
+						<< "}\n";
+			}
+			else {
+				os << "\\" << from_ascii(tclass.titlename())
+						<< "\n";
+			}
+			texrow.newline();
+			already_title = true;
+			was_title = false;
+		}
 
-			if (layout->is_environment) {
-				par = TeXOnePar(buf, text, par, os, texrow,
-						runparams, everypar);
-			} else if (layout->isEnvironment() ||
-				   !par->params().leftIndent().zero()) {
-				par = TeXEnvironment(buf, text, par, os,
-						     texrow, runparams);
-			} else {
-				par = TeXOnePar(buf, text, par, os, texrow,
-						runparams, everypar);
-			}
+		if (layout.isEnvironment() ||
+					!par->params().leftIndent().zero()) {
+			par = TeXEnvironment(buf, text, par, os,
+								texrow, runparams);
 		} else {
 			par = TeXOnePar(buf, text, par, os, texrow,
 					runparams, everypar);
 		}
-		if (std::distance(lastpar, par) >= std::distance(lastpar, endpar))
+		if (distance(lastpar, par) >= distance(lastpar, endpar))
 			break;
 	}
+
 	// It might be that we only have a title in this document
 	if (was_title && !already_title) {
 		if (tclass.titletype() == TITLE_ENVIRONMENT) {
@@ -841,6 +864,7 @@ void latexParagraphs(Buffer const & buf,
 				}
 		texrow.newline();
 	}
+
 	// if "auto end" is switched off, explicitely close the language at the end
 	// but only if the last par is in a babel language
 	if (maintext && !lyxrc.language_auto_end && !bparams.language->babel().empty() &&
@@ -851,6 +875,7 @@ void latexParagraphs(Buffer const & buf,
 			<< '\n';
 		texrow.newline();
 	}
+
 	// If the last paragraph is an environment, we'll have to close
 	// CJK at the very end to do proper nesting.
 	if (maintext && !is_child && open_encoding_ == CJK) {
@@ -858,6 +883,7 @@ void latexParagraphs(Buffer const & buf,
 		texrow.newline();
 		open_encoding_ = none;
 	}
+
 	// reset inherited encoding
 	if (cjk_inherited_ > 0) {
 		cjk_inherited_ -= 1;
@@ -868,12 +894,13 @@ void latexParagraphs(Buffer const & buf,
 
 
 pair<bool, int> switchEncoding(odocstream & os, BufferParams const & bparams,
-		   OutputParams const & runparams, Encoding const & newEnc)
+		   OutputParams const & runparams, Encoding const & newEnc,
+		   bool force)
 {
 	Encoding const & oldEnc = *runparams.encoding;
 	bool moving_arg = runparams.moving_arg;
-	if ((bparams.inputenc != "auto" && bparams.inputenc != "default")
-		|| moving_arg)
+	if (!force && ((bparams.inputenc != "auto" && bparams.inputenc != "default")
+		|| moving_arg))
 		return make_pair(false, 0);
 
 	// Do nothing if the encoding is unchanged.
@@ -889,9 +916,8 @@ pair<bool, int> switchEncoding(odocstream & os, BufferParams const & bparams,
 		|| newEnc.package() == Encoding::none)
 		return make_pair(false, 0);
 
-	LYXERR(Debug::LATEX) << "Changing LaTeX encoding from "
-		<< oldEnc.name() << " to "
-		<< newEnc.name() << endl;
+	LYXERR(Debug::LATEX, "Changing LaTeX encoding from "
+		<< oldEnc.name() << " to " << newEnc.name());
 	os << setEncoding(newEnc.iconvName());
 	if (bparams.inputenc == "default")
 		return make_pair(true, 0);
@@ -899,6 +925,7 @@ pair<bool, int> switchEncoding(odocstream & os, BufferParams const & bparams,
 	docstring const inputenc_arg(from_ascii(newEnc.latexName()));
 	switch (newEnc.package()) {
 		case Encoding::none:
+		case Encoding::japanese:
 			// shouldn't ever reach here, see above
 			return make_pair(true, 0);
 		case Encoding::inputenc: {
@@ -915,35 +942,42 @@ pair<bool, int> switchEncoding(odocstream & os, BufferParams const & bparams,
 				open_encoding_ = none;
 				count += 7;
 			}
-			if (runparams.local_font != 0 && oldEnc.package() == Encoding::CJK) {
-				// within insets, \inputenc switches need to be 
-				// embraced within \bgroup ... \egroup; else CJK fails.
+			if (runparams.local_font != 0
+			    && oldEnc.package() == Encoding::CJK) {
+				// within insets, \inputenc switches need
+				// to be embraced within \bgroup...\egroup;
+				// else CJK fails.
 				os << "\\bgroup";
 				count += 7;
 				open_encoding_ = inputenc;
 			}
+			// with the japanese option, inputenc is omitted.
+			if (runparams.use_japanese)
+				return make_pair(true, count);
 			os << "\\inputencoding{" << inputenc_arg << '}';
 			return make_pair(true, count + 16);
 		}
 		case Encoding::CJK: {
 			int count = inputenc_arg.length();
-			if (oldEnc.package() == Encoding::CJK && 
+			if (oldEnc.package() == Encoding::CJK &&
 			    open_encoding_ == CJK) {
 				os << "\\end{CJK}";
 				count += 9;
 			}
-			if (oldEnc.package() == Encoding::inputenc && 
+			if (oldEnc.package() == Encoding::inputenc &&
 			    open_encoding_ == inputenc) {
 				os << "\\egroup";
 				count += 7;
 			}
-			os << "\\begin{CJK}{" << inputenc_arg << "}{}";
+			os << "\\begin{CJK}{" << inputenc_arg << "}{"
+			   << from_ascii(bparams.fontsCJK) << "}";
 			open_encoding_ = CJK;
 			return make_pair(true, count + 15);
 		}
 	}
 	// Dead code to avoid a warning:
 	return make_pair(true, 0);
+
 }
 
 } // namespace lyx
