@@ -5,6 +5,7 @@
  *
  * \author Lars Gullik Bjnnes
  * \author Jean-Marc Lasgouttes
+ * \author Dekel Tsur
  *
  * Full author contact details are available in file CREDITS.
  */
@@ -12,40 +13,147 @@
 #include <config.h>
 
 #include "support/lstrings.h"
-#include "support/lyxlib.h"
+
 #include "support/convert.h"
 #include "support/qstring_helpers.h"
-
-#include "debug.h"
+#include "support/textutils.h"
 
 #include <boost/tokenizer.hpp>
-#include <boost/assert.hpp>
+#include "support/lassert.h"
 
-#ifndef I_AM_NOT_AFRAID_OF_HEADER_LIBRARIES
-#if USE_BOOST_FORMAT
-#include <boost/format.hpp>
-#endif
-#endif
-
-#include <cctype>
-#include <cstdlib>
+#include <QString>
+#include <QVector>
 
 #include <algorithm>
-#include <sstream>
-
-
-using std::transform;
-using std::string;
-using std::vector;
-
-#ifndef CXX_GLOBAL_CSTD
-using std::isdigit;
-using std::tolower;
-using std::toupper;
-#endif
 
+using namespace std;
 
 namespace lyx {
+
+// Using this allows us to have docstring default arguments in headers
+// without #include "support/docstring" there.
+docstring const & empty_docstring()
+{
+	static docstring s;
+	return s;
+}
+
+// Using this allows us to have string default arguments in headers
+// without #include <string>
+string const & empty_string()
+{
+	static string s;
+	return s;
+}
+
+/**
+ * Convert a QChar into a UCS4 character.
+ * This is a hack (it does only make sense for the common part of the UCS4
+ * and UTF16 encodings) and should not be used.
+ * This does only exist because of performance reasons (a real conversion
+ * using iconv is too slow on windows).
+ */
+static inline char_type qchar_to_ucs4(QChar const & qchar)
+{
+	LASSERT(is_utf16(static_cast<char_type>(qchar.unicode())), /**/);
+	return static_cast<char_type>(qchar.unicode());
+}
+
+
+/**
+ * Convert a UCS4 character into a QChar.
+ * This is a hack (it does only make sense for the common part of the UCS4
+ * and UTF16 encodings) and should not be used.
+ * This does only exist because of performance reasons (a real conversion
+ * using iconv is too slow on windows).
+ */
+static inline QChar const ucs4_to_qchar(char_type const ucs4)
+{
+	LASSERT(is_utf16(ucs4), /**/);
+	return QChar(static_cast<unsigned short>(ucs4));
+}
+
+
+namespace {
+	/// Maximum valid UCS4 code point
+	char_type const ucs4_max = 0x10ffff;
+}
+
+
+bool isLetterChar(char_type c)
+{
+	if (!is_utf16(c)) {
+		if (c > ucs4_max)
+			// outside the UCS4 range
+			return false;
+		// assume that all non-utf16 characters are letters
+		return true;
+	}
+	return ucs4_to_qchar(c).isLetter();
+}
+
+
+bool isAlphaASCII(char_type c)
+{
+	return ('a' <= c && c <= 'z') || ('A' <= c && c <= 'Z');
+}
+
+
+bool isPrintable(char_type c)
+{
+	if (!is_utf16(c)) {
+		if (c > ucs4_max)
+			// outside the UCS4 range
+			return false;
+		// assume that all non-utf16 characters are printable
+		return true;
+	}
+	return ucs4_to_qchar(c).isPrint();
+}
+
+
+bool isPrintableNonspace(char_type c)
+{
+	if (!is_utf16(c)) {
+		if (c > ucs4_max)
+			// outside the UCS4 range
+			return false;
+		// assume that all non-utf16 characters are printable and
+		// no space
+		return true;
+	}
+	QChar const qc = ucs4_to_qchar(c);
+	return qc.isPrint() && !qc.isSpace();
+}
+
+
+bool isSpace(char_type c)
+{
+	if (!is_utf16(c)) {
+		// assume that no non-utf16 character is a space
+		// c outside the UCS4 range is catched as well
+		return false;
+	}
+	QChar const qc = ucs4_to_qchar(c);
+	return qc.isSpace();
+}
+
+
+bool isDigit(char_type c)
+{
+	if (!is_utf16(c))
+		// assume that no non-utf16 character is a digit
+		// c outside the UCS4 range is catched as well
+		return false;
+	return ucs4_to_qchar(c).isDigit();
+}
+
+
+bool isDigitASCII(char_type c)
+{
+	return '0' <= c && c <= '9';
+}
+
 namespace support {
 
 int compare_no_case(docstring const & s, docstring const & s2)
@@ -128,73 +236,77 @@ int compare_ascii_no_case(docstring const & s, docstring const & s2)
 
 bool isStrInt(string const & str)
 {
-	if (str.empty()) return false;
+	if (str.empty())
+		return false;
 
 	// Remove leading and trailing white space chars.
 	string const tmpstr = trim(str);
-	if (tmpstr.empty()) return false;
+	if (tmpstr.empty())
+		return false;
 
 	string::const_iterator cit = tmpstr.begin();
-	if ((*cit) == '-') ++cit;
+	if ((*cit) == '-')
+		++cit;
+
 	string::const_iterator end = tmpstr.end();
-	for (; cit != end; ++cit) {
-		if (!isdigit((*cit))) return false;
-	}
+	for (; cit != end; ++cit)
+		if (!isdigit((*cit)))
+			return false;
+
 	return true;
 }
 
 
 bool isStrUnsignedInt(string const & str)
 {
-	if (str.empty()) return false;
+	if (str.empty())
+		return false;
 
 	// Remove leading and trailing white space chars.
 	string const tmpstr = trim(str);
-	if (tmpstr.empty()) return false;
+	if (tmpstr.empty())
+		return false;
 
 	string::const_iterator cit = tmpstr.begin();
 	string::const_iterator end = tmpstr.end();
-	for (; cit != end; ++cit) {
-		if (!isdigit((*cit))) return false;
-	}
+	for (; cit != end; ++cit)
+		if (!isdigit((*cit)))
+			return false;
+
 	return true;
 }
 
 
 bool isStrDbl(string const & str)
 {
-	if (str.empty()) return false;
+	if (str.empty())
+		return false;
 
 	// Remove leading and trailing white space chars.
 	string const tmpstr = trim(str);
-	if (tmpstr.empty()) return false;
-	//	if (1 < tmpstr.count('.')) return false;
+	if (tmpstr.empty())
+		return false;
+	//	if (tmpstr.count('.') > 1) return false;
 
 	string::const_iterator cit = tmpstr.begin();
-	bool found_dot(false);
-	if ((*cit) == '-') ++cit;
+	bool found_dot = false;
+	if (*cit == '-')
+		++cit;
 	string::const_iterator end = tmpstr.end();
 	for (; cit != end; ++cit) {
-		if (!isdigit((*cit))
-		    && '.' != (*cit)) {
+		if (!isdigit(*cit) && *cit != '.')
 			return false;
-		}
 		if ('.' == (*cit)) {
-			if (found_dot) {
+			if (found_dot)
 				return false;
-			} else {
-				found_dot = true;
-			}
+			found_dot = true;
 		}
 	}
 	return true;
 }
 
 
-namespace {
-
-inline
-bool isHexChar(char_type c)
+static bool isHexChar(char_type c)
 {
 	return c == '0' ||
 		c == '1' ||
@@ -214,8 +326,6 @@ bool isHexChar(char_type c)
 		c == 'f' || c == 'F';
 }
 
-} // anon namespace
-
 
 bool isHex(docstring const & str)
 {
@@ -266,14 +376,14 @@ bool isAscii(string const & str)
 
 char lowercase(char c)
 {
-	BOOST_ASSERT(static_cast<unsigned char>(c) < 0x80);
+	LASSERT(static_cast<unsigned char>(c) < 0x80, /**/);
 	return char(tolower(c));
 }
 
 
 char uppercase(char c)
 {
-	BOOST_ASSERT(static_cast<unsigned char>(c) < 0x80);
+	LASSERT(static_cast<unsigned char>(c) < 0x80, /**/);
 	return char(toupper(c));
 }
 
@@ -298,8 +408,8 @@ char_type uppercase(char_type c)
 
 namespace {
 
-// since we cannot use std::tolower and std::toupper directly in the
-// calls to std::transform yet, we use these helper clases. (Lgb)
+// since we cannot use tolower and toupper directly in the
+// calls to transform yet, we use these helper clases. (Lgb)
 
 struct local_lowercase {
 	char_type operator()(char_type c) const {
@@ -320,9 +430,7 @@ struct local_uppercase {
 };
 
 template<typename Char> struct local_ascii_lowercase {
-	Char operator()(Char c) const {
-		return ascii_tolower(c);
-	}
+	Char operator()(Char c) const { return ascii_tolower(c); }
 };
 
 } // end of anon namespace
@@ -371,36 +479,24 @@ bool prefixIs(docstring const & a, char_type c)
 
 bool prefixIs(string const & a, string const & pre)
 {
-	string::size_type const prelen = pre.length();
-	string::size_type const alen = a.length();
-
-	if (prelen > alen || a.empty())
-		return false;
-	else {
-#if defined(STD_STRING_IS_GOOD)
-		return a.compare(0, prelen, pre) == 0;
-#else
-		return ::strncmp(a.c_str(), pre.c_str(), prelen) == 0;
-#endif
-	}
+	size_t const prelen = pre.length();
+	size_t const alen = a.length();
+	return prelen <= alen && !a.empty() && a.compare(0, prelen, pre) == 0;
 }
 
 
 bool prefixIs(docstring const & a, docstring const & pre)
 {
-	docstring::size_type const prelen = pre.length();
-	docstring::size_type const alen = a.length();
-
-	if (prelen > alen || a.empty())
-		return false;
-	else
-		return a.compare(0, prelen, pre) == 0;
+	size_t const prelen = pre.length();
+	size_t const alen = a.length();
+	return prelen <= alen && !a.empty() && a.compare(0, prelen, pre) == 0;
 }
 
 
 bool suffixIs(string const & a, char c)
 {
-	if (a.empty()) return false;
+	if (a.empty()) 
+		return false;
 	return a[a.length() - 1] == c;
 }
 
@@ -415,19 +511,17 @@ bool suffixIs(docstring const & a, char_type c)
 
 bool suffixIs(string const & a, string const & suf)
 {
-	string::size_type const suflen = suf.length();
-	string::size_type const alen = a.length();
+	size_t const suflen = suf.length();
+	size_t const alen = a.length();
+	return suflen <= alen && a.compare(alen - suflen, suflen, suf) == 0;
+}
 
-	if (suflen > alen) {
-		return false;
-	} else {
-#if !defined(USE_INCLUDED_STRING) && !defined(STD_STRING_IS_GOOD)
-		string tmp(a, alen - suflen);
-		return ::strncmp(tmp.c_str(), suf.c_str(), suflen) == 0;
-#else
-		return a.compare(alen - suflen, suflen, suf) == 0;
-#endif
-	}
+
+bool suffixIs(docstring const & a, docstring const & suf)
+{
+	size_t const suflen = suf.length();
+	size_t const alen = a.length();
+	return suflen <= alen && a.compare(alen - suflen, suflen, suf) == 0;
 }
 
 
@@ -441,19 +535,24 @@ bool containsOnly(string const & s, string const & cset)
 // rewritten to use new string (Lgb)
 string const token(string const & a, char delim, int n)
 {
-	if (a.empty()) return string();
+	if (a.empty())
+		return string();
 
-	string::size_type k = 0;
-	string::size_type i = 0;
+	size_t k = 0;
+	size_t i = 0;
 
 	// Find delimiter or end of string
-	for (; n--;)
+	for (; n--;) {
 		if ((i = a.find(delim, i)) == string::npos)
 			break;
 		else
 			++i; // step delim
+	}
+
 	// i is now the n'th delim (or string::npos)
-	if (i == string::npos) return string();
+	if (i == string::npos)
+		return string();
+
 	k = a.find(delim, i);
 	// k is now the n'th + 1 delim (or string::npos)
 
@@ -463,19 +562,24 @@ string const token(string const & a, char delim, int n)
 
 docstring const token(docstring const & a, char_type delim, int n)
 {
-	if (a.empty()) return docstring();
+	if (a.empty())
+		return docstring();
 
-	string::size_type k = 0;
-	string::size_type i = 0;
+	size_t k = 0;
+	size_t i = 0;
 
 	// Find delimiter or end of string
-	for (; n--;)
+	for (; n--;) {
 		if ((i = a.find(delim, i)) == docstring::npos)
 			break;
 		else
 			++i; // step delim
+	}
+
 	// i is now the n'th delim (or string::npos)
-	if (i == docstring::npos) return docstring();
+	if (i == docstring::npos)
+		return docstring();
+
 	k = a.find(delim, i);
 	// k is now the n'th + 1 delim (or string::npos)
 
@@ -488,7 +592,7 @@ docstring const token(docstring const & a, char_type delim, int n)
 int tokenPos(string const & a, char delim, string const & tok)
 {
 	int i = 0;
-	string str(a);
+	string str = a;
 	string tmptok;
 
 	while (!str.empty()) {
@@ -501,14 +605,32 @@ int tokenPos(string const & a, char delim, string const & tok)
 }
 
 
+// this could probably be faster and/or cleaner, but it seems to work (JMarc)
+// rewritten to use new string (Lgb)
+int tokenPos(docstring const & a, char_type delim, docstring const & tok)
+{
+	int i = 0;
+	docstring str = a;
+	docstring tmptok;
+
+	while (!str.empty()) {
+		str = split(str, tmptok, delim);
+		if (tok == tmptok)
+			return i;
+		++i;
+	}
+	return -1;
+}
+
+
 namespace {
 
 /// Substitute all \a oldchar with \a newchar
 template<typename Ch> inline
-std::basic_string<Ch> const subst_char(std::basic_string<Ch> const & a,
+basic_string<Ch> const subst_char(basic_string<Ch> const & a,
 		Ch oldchar, Ch newchar)
 {
-	typedef std::basic_string<Ch> String;
+	typedef basic_string<Ch> String;
 	String tmp(a);
 	typename String::iterator lit = tmp.begin();
 	typename String::iterator end = tmp.end();
@@ -518,16 +640,44 @@ std::basic_string<Ch> const subst_char(std::basic_string<Ch> const & a,
 	return tmp;
 }
 
+/// Substitute all \a oldchar with \a newchar
+docstring const subst_char(docstring const & a,
+	docstring::value_type oldchar, docstring::value_type newchar)
+{
+	docstring tmp(a);
+	docstring::iterator lit = tmp.begin();
+	docstring::iterator end = tmp.end();
+	for (; lit != end; ++lit)
+		if ((*lit) == oldchar)
+			(*lit) = newchar;
+	return tmp;
+}
+
 
 /// substitutes all instances of \a oldstr with \a newstr
 template<typename String> inline
 String const subst_string(String const & a,
 		String const & oldstr, String const & newstr)
 {
-	BOOST_ASSERT(!oldstr.empty());
+	LASSERT(!oldstr.empty(), /**/);
 	String lstr = a;
-	typename String::size_type i = 0;
-	typename String::size_type const olen = oldstr.length();
+	size_t i = 0;
+	size_t const olen = oldstr.length();
+	while ((i = lstr.find(oldstr, i)) != string::npos) {
+		lstr.replace(i, olen, newstr);
+		i += newstr.length(); // We need to be sure that we dont
+		// use the same i over and over again.
+	}
+	return lstr;
+}
+
+docstring const subst_string(docstring const & a,
+		docstring const & oldstr, docstring const & newstr)
+{
+	LASSERT(!oldstr.empty(), /**/);
+	docstring lstr = a;
+	size_t i = 0;
+	size_t const olen = oldstr.length();
 	while ((i = lstr.find(oldstr, i)) != string::npos) {
 		lstr.replace(i, olen, newstr);
 		i += newstr.length(); // We need to be sure that we dont
@@ -568,14 +718,14 @@ docstring const subst(docstring const & a,
 
 docstring const trim(docstring const & a, char const * p)
 {
-	BOOST_ASSERT(p);
+	LASSERT(p, /**/);
 
 	if (a.empty() || !*p)
 		return a;
 
-	docstring s = lyx::from_ascii(p);
-	docstring::size_type r = a.find_last_not_of(s);
-	docstring::size_type l = a.find_first_not_of(s);
+	docstring s = from_ascii(p);
+	size_t r = a.find_last_not_of(s);
+	size_t l = a.find_first_not_of(s);
 
 	// Is this the minimal test? (lgb)
 	if (r == docstring::npos && l == docstring::npos)
@@ -587,13 +737,13 @@ docstring const trim(docstring const & a, char const * p)
 
 string const trim(string const & a, char const * p)
 {
-	BOOST_ASSERT(p);
+	LASSERT(p, /**/);
 
 	if (a.empty() || !*p)
 		return a;
 
-	string::size_type r = a.find_last_not_of(p);
-	string::size_type l = a.find_first_not_of(p);
+	size_t r = a.find_last_not_of(p);
+	size_t l = a.find_first_not_of(p);
 
 	// Is this the minimal test? (lgb)
 	if (r == string::npos && l == string::npos)
@@ -605,12 +755,12 @@ string const trim(string const & a, char const * p)
 
 string const rtrim(string const & a, char const * p)
 {
-	BOOST_ASSERT(p);
+	LASSERT(p, /**/);
 
 	if (a.empty() || !*p)
 		return a;
 
-	string::size_type r = a.find_last_not_of(p);
+	size_t r = a.find_last_not_of(p);
 
 	// Is this test really needed? (Lgb)
 	if (r == string::npos)
@@ -622,12 +772,12 @@ string const rtrim(string const & a, char const * p)
 
 docstring const rtrim(docstring const & a, char const * p)
 {
-	BOOST_ASSERT(p);
+	LASSERT(p, /**/);
 
 	if (a.empty() || !*p)
 		return a;
 
-	docstring::size_type r = a.find_last_not_of(from_ascii(p));
+	size_t r = a.find_last_not_of(from_ascii(p));
 
 	// Is this test really needed? (Lgb)
 	if (r == docstring::npos)
@@ -639,10 +789,10 @@ docstring const rtrim(docstring const & a, char const * p)
 
 string const ltrim(string const & a, char const * p)
 {
-	BOOST_ASSERT(p);
+	LASSERT(p, /**/);
 	if (a.empty() || !*p)
 		return a;
-	string::size_type l = a.find_first_not_of(p);
+	size_t l = a.find_first_not_of(p);
 	if (l == string::npos)
 		return string();
 	return a.substr(l, string::npos);
@@ -651,7 +801,7 @@ string const ltrim(string const & a, char const * p)
 
 docstring const ltrim(docstring const & a, char const * p)
 {
-	BOOST_ASSERT(p);
+	LASSERT(p, /**/);
 	if (a.empty() || !*p)
 		return a;
 	size_t l = a.find_first_not_of(from_ascii(p));
@@ -666,7 +816,7 @@ template<typename String, typename Char> inline
 String const doSplit(String const & a, String & piece, Char delim)
 {
 	String tmp;
-	typename String::size_type i = a.find(delim);
+	size_t i = a.find(delim);
 	if (i == a.length() - 1) {
 		piece = a.substr(0, i);
 	} else if (i != String::npos) {
@@ -681,8 +831,27 @@ String const doSplit(String const & a, String & piece, Char delim)
 	return tmp;
 }
 
+template<typename Char> inline
+docstring const doSplit(docstring const & a, docstring & piece, Char delim)
+{
+	docstring tmp;
+	size_t i = a.find(delim);
+	if (i == a.length() - 1) {
+		piece = a.substr(0, i);
+	} else if (i != docstring::npos) {
+		piece = a.substr(0, i);
+		tmp = a.substr(i + 1);
+	} else if (i == 0) {
+		piece.erase();
+		tmp = a.substr(i + 1);
+	} else {
+		piece = a;
+	}
+	return tmp;
 }
 
+} // anon
+
 
 string const split(string const & a, string & piece, char delim)
 {
@@ -699,7 +868,7 @@ docstring const split(docstring const & a, docstring & piece, char_type delim)
 string const split(string const & a, char delim)
 {
 	string tmp;
-	string::size_type i = a.find(delim);
+	size_t i = a.find(delim);
 	if (i != string::npos) // found delim
 		tmp = a.substr(i + 1);
 	return tmp;
@@ -710,7 +879,7 @@ string const split(string const & a, char delim)
 string const rsplit(string const & a, string & piece, char delim)
 {
 	string tmp;
-	string::size_type i = a.rfind(delim);
+	size_t i = a.rfind(delim);
 	if (i != string::npos) { // delimiter was found
 		piece = a.substr(0, i);
 		tmp = a.substr(i + 1);
@@ -726,7 +895,7 @@ docstring const escape(docstring const & lab)
 	char_type hexdigit[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
 				   '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
 	docstring enc;
-	for (docstring::size_type i = 0; i < lab.length(); ++i) {
+	for (size_t i = 0; i < lab.length(); ++i) {
 		char_type c = lab[i];
 		if (c >= 128 || c == '=' || c == '%') {
 			// Although char_type is a 32 bit type we know that
@@ -734,7 +903,7 @@ docstring const escape(docstring const & lab)
 			// encode bigger values. Test for 2^24 because we
 			// can encode that with the 6 hex digits that are
 			// needed for 21 bits anyway.
-			BOOST_ASSERT(c < (1 << 24));
+			LASSERT(c < (1 << 24), /**/);
 			enc += '=';
 			enc += hexdigit[(c>>20) & 15];
 			enc += hexdigit[(c>>16) & 15];
@@ -753,7 +922,7 @@ docstring const escape(docstring const & lab)
 namespace {
 
 template<typename String> vector<String> const
-getVectorFromStringT(String const & str, String const & delim)
+getVectorFromStringT(String const & str, String const & delim, bool keepempty)
 {
 // Lars would like this code to go, but for now his replacement (below)
 // doesn't fullfil the same function. I have, therefore, reactivated the
@@ -763,16 +932,16 @@ getVectorFromStringT(String const & str, String const & delim)
 	if (str.empty())
 		return vec;
 	String keys = rtrim(str);
-	for(;;) {
-		typename String::size_type const idx = keys.find(delim);
+	while (true) {
+		size_t const idx = keys.find(delim);
 		if (idx == String::npos) {
 			vec.push_back(ltrim(keys));
 			break;
 		}
 		String const key = trim(keys.substr(0, idx));
-		if (!key.empty())
+		if (!key.empty() || keepempty)
 			vec.push_back(key);
-		typename String::size_type const start = idx + delim.size();
+		size_t const start = idx + delim.size();
 		keys = keys.substr(start);
 	}
 	return vec;
@@ -785,20 +954,22 @@ getVectorFromStringT(String const & str, String const & delim)
 #endif
 }
 
-}
+} // namespace anon
 
 
 vector<string> const getVectorFromString(string const & str,
-					 string const & delim)
+					 string const & delim,
+					 bool keepempty)
 {
-	return getVectorFromStringT<string>(str, delim);
+	return getVectorFromStringT<string>(str, delim, keepempty);
 }
 
 
 vector<docstring> const getVectorFromString(docstring const & str,
-					    docstring const & delim)
+					    docstring const & delim,
+					    bool keepempty)
 {
-	return getVectorFromStringT<docstring>(str, delim);
+	return getVectorFromStringT<docstring>(str, delim, keepempty);
 }
 
 
@@ -833,191 +1004,94 @@ int findToken(char const * const str[], string const & search_token)
 }
 
 
-docstring const externalLineEnding(docstring const & str)
-{
-#if defined(__APPLE__)
-	// The MAC clipboard uses \r for lineendings, and we use \n
-	return subst(str, '\n', '\r');
-#elif defined (_WIN32) || (defined (__CYGWIN__) && defined (X_DISPLAY_MISSING))
-	// Windows clipboard uses \r\n for lineendings, and we use \n
-	return subst(str, lyx::from_ascii("\n"), lyx::from_ascii("\r\n"));
-#else
-	return str;
-#endif
-}
-
-
-docstring const internalLineEnding(docstring const & str)
-{
-	docstring const s = subst(str,
-			lyx::from_ascii("\r\n"), lyx::from_ascii("\n"));
-	return subst(s, '\r', '\n');
-}
-
-
-#ifndef I_AM_NOT_AFRAID_OF_HEADER_LIBRARIES
-#if USE_BOOST_FORMAT
-
 template<>
 docstring bformat(docstring const & fmt, int arg1)
 {
-	return (boost::basic_format<char_type>(fmt) % arg1).str();
+	LASSERT(contains(fmt, from_ascii("%1$d")), /**/);
+	docstring const str = subst(fmt, from_ascii("%1$d"), convert<docstring>(arg1));
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
 
 template<>
 docstring bformat(docstring const & fmt, long arg1)
 {
-	return (boost::basic_format<char_type>(fmt) % arg1).str();
+	LASSERT(contains(fmt, from_ascii("%1$d")), /**/);
+	docstring const str = subst(fmt, from_ascii("%1$d"), convert<docstring>(arg1));
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
 
 template<>
 docstring bformat(docstring const & fmt, unsigned int arg1)
 {
-	return (boost::basic_format<char_type>(fmt) % arg1).str();
-}
-
-
-template<>
-docstring bformat<docstring>(docstring const & fmt, docstring arg1)
-{
-	return (boost::basic_format<char_type>(fmt) % arg1).str();
-}
-
-
-template<>
-docstring bformat(docstring const & fmt, char * arg1)
-{
-	return (boost::basic_format<char_type>(fmt) % arg1).str();
-}
-
-
-template<>
-docstring bformat(docstring const & fmt, int arg1, int arg2)
-{
-	return (boost::basic_format<char_type>(fmt) % arg1 % arg2).str();
-}
-
-
-template<>
-docstring bformat(docstring const & fmt, docstring arg1, docstring arg2)
-{
-	return (boost::basic_format<char_type>(fmt) % arg1 % arg2).str();
-}
-
-
-template<>
-docstring bformat(docstring const & fmt, char const * arg1, docstring arg2)
-{
-	return (boost::basic_format<char_type>(fmt) % arg1 % arg2).str();
-}
-
-
-template<>
-docstring bformat(docstring const & fmt, docstring arg1, docstring arg2, docstring arg3)
-{
-	return (boost::basic_format<char_type>(fmt) % arg1 % arg2 % arg3).str();
-}
-
-
-template<>
-docstring bformat(docstring const & fmt,
-	       docstring arg1, docstring arg2, docstring arg3, docstring arg4)
-{
-	return (boost::basic_format<char_type>(fmt) % arg1 % arg2 % arg3 % arg4).str();
-}
-
-#else
-
-template<>
-docstring bformat(docstring const & fmt, int arg1)
-{
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$d")));
-	docstring const str = subst(fmt, lyx::from_ascii("%1$d"), convert<docstring>(arg1));
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
-}
-
-
-template<>
-docstring bformat(docstring const & fmt, long arg1)
-{
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$d")));
-	docstring const str = subst(fmt, lyx::from_ascii("%1$d"), convert<docstring>(arg1));
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
-}
-
-
-template<>
-docstring bformat(docstring const & fmt, unsigned int arg1)
-{
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$d")));
-	docstring const str = subst(fmt, lyx::from_ascii("%1$d"), convert<docstring>(arg1));
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
+	LASSERT(contains(fmt, from_ascii("%1$d")), /**/);
+	docstring const str = subst(fmt, from_ascii("%1$d"), convert<docstring>(arg1));
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
 
 template<>
 docstring bformat(docstring const & fmt, docstring arg1)
 {
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$s")));
-	docstring const str = subst(fmt, lyx::from_ascii("%1$s"), arg1);
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
+	LASSERT(contains(fmt, from_ascii("%1$s")), /**/);
+	docstring const str = subst(fmt, from_ascii("%1$s"), arg1);
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
 
 template<>
 docstring bformat(docstring const & fmt, char * arg1)
 {
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$s")));
-	docstring const str = subst(fmt, lyx::from_ascii("%1$s"), lyx::from_ascii(arg1));
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
+	LASSERT(contains(fmt, from_ascii("%1$s")), /**/);
+	docstring const str = subst(fmt, from_ascii("%1$s"), from_ascii(arg1));
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
 
 template<>
 docstring bformat(docstring const & fmt, docstring arg1, docstring arg2)
 {
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$s")));
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%2$s")));
-	docstring str = subst(fmt, lyx::from_ascii("%1$s"), arg1);
-	str = subst(str, lyx::from_ascii("%2$s"), arg2);
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
+	LASSERT(contains(fmt, from_ascii("%1$s")), /**/);
+	LASSERT(contains(fmt, from_ascii("%2$s")), /**/);
+	docstring str = subst(fmt, from_ascii("%1$s"), arg1);
+	str = subst(str, from_ascii("%2$s"), arg2);
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
 
 template<>
 docstring bformat(docstring const & fmt, char const * arg1, docstring arg2)
 {
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$s")));
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%2$s")));
-	docstring str = subst(fmt, lyx::from_ascii("%1$s"), lyx::from_ascii(arg1));
-	str = subst(fmt, lyx::from_ascii("%2$s"), arg2);
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
+	LASSERT(contains(fmt, from_ascii("%1$s")), /**/);
+	LASSERT(contains(fmt, from_ascii("%2$s")), /**/);
+	docstring str = subst(fmt, from_ascii("%1$s"), from_ascii(arg1));
+	str = subst(fmt, from_ascii("%2$s"), arg2);
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
 
 template<>
 docstring bformat(docstring const & fmt, int arg1, int arg2)
 {
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$d")));
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%2$d")));
-	docstring str = subst(fmt, lyx::from_ascii("%1$d"), convert<docstring>(arg1));
-	str = subst(str, lyx::from_ascii("%2$d"), convert<docstring>(arg2));
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
+	LASSERT(contains(fmt, from_ascii("%1$d")), /**/);
+	LASSERT(contains(fmt, from_ascii("%2$d")), /**/);
+	docstring str = subst(fmt, from_ascii("%1$d"), convert<docstring>(arg1));
+	str = subst(str, from_ascii("%2$d"), convert<docstring>(arg2));
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
 
 template<>
 docstring bformat(docstring const & fmt, docstring arg1, docstring arg2, docstring arg3)
 {
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$s")));
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%2$s")));
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%3$s")));
-	docstring str = subst(fmt, lyx::from_ascii("%1$s"), arg1);
-	str = subst(str, lyx::from_ascii("%2$s"), arg2);
-	str = subst(str, lyx::from_ascii("%3$s"), arg3);
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
+	LASSERT(contains(fmt, from_ascii("%1$s")), /**/);
+	LASSERT(contains(fmt, from_ascii("%2$s")), /**/);
+	LASSERT(contains(fmt, from_ascii("%3$s")), /**/);
+	docstring str = subst(fmt, from_ascii("%1$s"), arg1);
+	str = subst(str, from_ascii("%2$s"), arg2);
+	str = subst(str, from_ascii("%3$s"), arg3);
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
 
@@ -1025,19 +1099,16 @@ template<>
 docstring bformat(docstring const & fmt,
 	       docstring arg1, docstring arg2, docstring arg3, docstring arg4)
 {
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%1$s")));
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%2$s")));
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%3$s")));
-	BOOST_ASSERT(contains(fmt, lyx::from_ascii("%4$s")));
-	docstring str = subst(fmt, lyx::from_ascii("%1$s"), arg1);
-	str = subst(str, lyx::from_ascii("%2$s"), arg2);
-	str = subst(str, lyx::from_ascii("%3$s"), arg3);
-	str = subst(str, lyx::from_ascii("%4$s"), arg4);
-	return subst(str, lyx::from_ascii("%%"), lyx::from_ascii("%"));
+	LASSERT(contains(fmt, from_ascii("%1$s")), /**/);
+	LASSERT(contains(fmt, from_ascii("%2$s")), /**/);
+	LASSERT(contains(fmt, from_ascii("%3$s")), /**/);
+	LASSERT(contains(fmt, from_ascii("%4$s")), /**/);
+	docstring str = subst(fmt, from_ascii("%1$s"), arg1);
+	str = subst(str, from_ascii("%2$s"), arg2);
+	str = subst(str, from_ascii("%3$s"), arg3);
+	str = subst(str, from_ascii("%4$s"), arg4);
+	return subst(str, from_ascii("%%"), from_ascii("%"));
 }
 
-#endif
-#endif
-
 } // namespace support
 } // namespace lyx
